@@ -39,7 +39,7 @@ def image_data_format():
 
 
 class KerasCallback(Callback):
-    def __init__(self, job_backend, general_logger, force_insights=False):
+    def __init__(self, job_backend, logger, force_insights=False):
         self.params = {}
         super(KerasCallback, self).__init__()
         self.validation_per_batch = []
@@ -55,7 +55,7 @@ class KerasCallback(Callback):
         self.confusion_matrix = True
 
         self.job_backend = job_backend
-        self.general_logger = general_logger
+        self.logger = logger
         self._test_with_acc = None
         self.last_batch_time = time.time()
         self.start_time = time.time()
@@ -127,6 +127,9 @@ class KerasCallback(Callback):
         if self.data_validation_size is None:
             raise Exception('data_validation_size could not be determined for given validation_data. Please specify it.')
 
+    def on_train_end(self, logs={}):
+        self.job_backend.sync_weights()
+
     def on_train_begin(self, logs={}):
         self.start_time = time.time()
         self.last_batch_time = time.time()
@@ -134,6 +137,7 @@ class KerasCallback(Callback):
         self.job_backend.set_info('parameters', get_total_params(self.model))
         self.job_backend.set_info('backend', K.backend())
         self.job_backend.set_info('Keras version', keras.__version__)
+        self.job_backend.set_info('Keras floatx', K.floatx())
 
         if hasattr(K, 'image_dim_ordering'):
             self.job_backend.set_info('Keras image_data_format', K.image_dim_ordering())
@@ -166,8 +170,8 @@ class KerasCallback(Callback):
         if len(self.model.output_layers) > 1:
             traces = []
             for output in self.model.output_layers:
-                traces.append(output.name+'_training')
-                traces.append(output.name+'_validation')
+                traces.append('train_' + output.name)
+                traces.append('val_' + output.name)
 
         is_accuracy_kpi = True if self.job_backend.kpi_channel is None else False
 
@@ -183,8 +187,8 @@ class KerasCallback(Callback):
         if len(self.model.output_layers) > 1:
             loss_traces = []
             for output in self.model.output_layers:
-                loss_traces.append(output.name+'_validation')
-                loss_traces.append(output.name+'_training')
+                loss_traces.append('train_' + output.name)
+                loss_traces.append('val_' + output.name)
 
             self.all_losses = self.job_backend.create_channel('All loss', main=True, xaxis=xaxis, traces=loss_traces)
 
@@ -213,12 +217,11 @@ class KerasCallback(Callback):
         self.job_backend.batch(batch, self.current['nb_batches'], logs['size'])
 
     def write(self, line):
-        self.general_logger.write(line)
+        self.logger.info(line)
 
     def on_epoch_begin(self, epoch, logs={}):
         self.learning_rate_start = self.get_learning_rate()
 
-
     def on_epoch_end(self, epoch, logs={}):
         log = logs.copy()
 
@@ -232,43 +235,39 @@ class KerasCallback(Callback):
         accuracy_log_name = 'acc'
         val_accuracy_log_name = 'val_acc'
 
-        if len(self.model.output_layers) > 1:
-            name = self.model.output_layers[0].name
-            accuracy_log_name = name+'_acc'
-            val_accuracy_log_name = 'val_' + name + '_acc'
-
         total_accuracy_validation = log.get(val_accuracy_log_name, 0)
         total_accuracy_training = log.get(accuracy_log_name, 0)
 
-        if total_accuracy_validation > self.best_total_accuracy:
-            self.best_total_accuracy = total_accuracy_validation
-            self.best_epoch = log['epoch']
-            try:
-                self.model.save_weights(self.filepath_best, overwrite=True)
-            except:
-                # sometimes hangs with: IOError: Unable to create file (Unable to open file: name = ...
-                # without any obvious reason.
-                pass
+        # todo, make this optional
+        # if total_accuracy_validation > self.best_total_accuracy:
+        #     self.best_total_accuracy = total_accuracy_validation
+        #     self.best_epoch = log['epoch']
+        #     try:
+        #         self.model.save_weights(self.filepath_best, overwrite=True)
+        #     except:
+        #         # sometimes hangs with: IOError: Unable to create file (Unable to open file: name = ...
+        #         # without any obvious reason.
+        #         pass
 
         try:
             self.model.save_weights(self.filepath_latest, overwrite=True)
-        except:
+        except Exception:
             # sometimes hangs with: IOError: Unable to create file (Unable to open file: name = ...
             # without any obvious reason.
             pass
 
         self.loss_channel.send(log['epoch'], log.get('loss', 0), log.get('val_loss', 0))
 
-        accuracy = [total_accuracy_validation*100, total_accuracy_training*100]
+        accuracy = [total_accuracy_training*100, total_accuracy_validation*100]
         if len(self.model.output_layers) > 1:
             accuracy = []
             losses = []
             for layer in self.model.output_layers:
-                accuracy.append(log.get('val_' + layer.name + '_acc', 0)*100)
                 accuracy.append(log.get(layer.name + '_acc', 0)*100)
+                accuracy.append(log.get('val_' + layer.name + '_acc', 0)*100)
 
-                losses.append(log.get('val_' + layer.name + '_loss', 0))
                 losses.append(log.get(layer.name + '_loss', 0))
+                losses.append(log.get('val_' + layer.name + '_loss', 0))
 
             self.all_losses.send(log['epoch'], losses)
 
@@ -278,13 +277,13 @@ class KerasCallback(Callback):
         self.send_optimizer_info(log['epoch'])
 
         if self.log_epoch:
-            #todo, multiple outputs
+            # todo, multiple outputs
             line = "Epoch %d: loss=%f, acc=%f, val_loss=%f, val_acc=%f\n" % (
                 log['epoch'], log['loss'], log.get('acc', 0), log.get('val_loss', 0), total_accuracy_validation, )
-            self.general_logger.write(line)
+            self.logger.write(line)
 
-        if self.force_insights or self.job_model.config['insights']:
-            # Todo, support multiple inputs
+        if self.force_insights or self.job_model.insights_enabled:
+            # todo, support multiple inputs
             first_input_layer = self.model.input_layers[0]
 
             if first_input_layer is not None:
@@ -350,7 +349,10 @@ class KerasCallback(Callback):
 
         for i, layer in enumerate(self.model.input_layers):
             x = input_data_x_sample[i][0]
-            if self.is_image_shape(x):
+            if len(x.shape) == 3 and self.is_image_shape(x):
+                if K.image_dim_ordering() == 'tf':
+                    x = np.transpose(x, (2, 0, 1))
+
                 image = self.make_image(x)
                 if image:
                     images.append(JobImage(layer.name, image))
@@ -364,9 +366,10 @@ class KerasCallback(Callback):
 
         layers = self.model.layers + self.insight_layer
 
+        pos = 0
         for layer in layers:
-            if isinstance(layer, keras.layers.convolutional.Convolution2D) or isinstance(layer, keras.layers.convolutional.MaxPooling2D):
-
+            if isinstance(layer, keras.layers.convolutional.Convolution2D) or isinstance(layer, keras.layers.convolutional.MaxPooling2D)\
+                    or isinstance(layer, keras.layers.convolutional.UpSampling2D):
                 fn = K.function(inputs, self.get_layout_output_tensors(layer))
                 Y = fn(input_data_x_sample)[0]
 
@@ -377,7 +380,8 @@ class KerasCallback(Callback):
                         data = np.transpose(data, (2, 0, 1))
 
                     image = PIL.Image.fromarray(get_image_tales(data))
-                    images.append(JobImage(layer.name, image))
+                    pos += 1
+                    images.append(JobImage(layer.name, image, pos=pos))
 
                 if layer.get_weights():
                     data = layer.get_weights()[0]
@@ -396,7 +400,8 @@ class KerasCallback(Callback):
                     data = data.reshape((data.shape[0] * data.shape[1], data.shape[2], data.shape[3]))
 
                     image = PIL.Image.fromarray(get_image_tales(data))
-                    images.append(JobImage(layer.name + '_weights', image, layer.name + ' weights'))
+                    pos += 1
+                    images.append(JobImage(layer.name + '_weights', image, layer.name + ' weights', pos=pos))
 
             elif isinstance(layer, keras.layers.ZeroPadding2D) or isinstance(layer, keras.layers.ZeroPadding1D) or isinstance(layer, keras.layers.ZeroPadding3D):
                 pass
@@ -415,11 +420,9 @@ class KerasCallback(Callback):
                     if Y.size == 1:
                         Y = np.array([Y])
 
-
                     image = None
                     if len(Y.shape) > 1:
-
-                        if len(Y.shape) == 3 and isinstance(layer, keras.layers.Activation) and K.image_dim_ordering() == 'tf':
+                        if len(Y.shape) == 3 and self.is_image_shape(Y) and K.image_dim_ordering() == 'tf':
                             Y = np.transpose(Y, (2, 0, 1))
 
                         image = PIL.Image.fromarray(get_layer_vis_square(Y))
@@ -427,7 +430,8 @@ class KerasCallback(Callback):
                         image = self.make_image_from_dense(Y)
 
                     if image:
-                        images.append(JobImage(layer.name, image))
+                        pos += 1
+                        images.append(JobImage(layer.name, image, pos=pos))
 
         return images
 
@@ -525,8 +529,7 @@ class KerasCallback(Callback):
                     for sample_idx, predicted_class in enumerate(predicted_classes):
                         expected_class = expected_classes[sample_idx]
                         matrix[expected_class, predicted_class] += 1
-                except:
-                    pass
+                except Exception: pass
 
         else:
             batch_size = self.current['batch_size'] if 'batch_size' in self.current else 16
@@ -538,8 +541,7 @@ class KerasCallback(Callback):
                 for sample_idx, predicted_class in enumerate(predicted_classes):
                     expected_class = expected_classes[sample_idx]
                     matrix[expected_class, predicted_class] += 1
-            except:
-                pass
+            except Exception: pass
 
         confusion_matrix[first_output_layer.name] = matrix.tolist()
 
@@ -560,7 +562,7 @@ class KerasCallback(Callback):
                 data = data.reshape((data.shape[0], data.shape[1], 1))
 
             image = array_to_img(data)
-        except:
+        except Exception:
             return None
 
         # image = image.resize((128, 128))
