@@ -18,6 +18,11 @@ command_summaries = [
     ['run', 'Executes a command on an AETROS server.'],
     ['api', 'Executes a API call through SSH connection.'],
     ['push-job', 'Pushes a local job to AETROS Trainer.'],
+    ['pull-job', 'Pulls a job to local Git repository.'],
+    ['add', 'Adds a local file to a job tree.'],
+    ['init', 'Creates a new model and places a aetros.yml in current working directory pointing to this model.'],
+    ['id', 'Shows under which account the machine is authenticated.'],
+    ['gpu', 'Shows information about installed GPUs'],
 ]
 
 def parseopts(args):
@@ -49,43 +54,51 @@ def main(args=None):
 
     from aetros.commands.ApiCommand import ApiCommand
     from aetros.commands.PushJobCommand import PushJobCommand
+    from aetros.commands.PullJobCommand import PullJobCommand
     from aetros.commands.ServerCommand import ServerCommand
-    from aetros.commands.UploadWeightsCommand import UploadWeightsCommand
     from aetros.commands.PredictCommand import PredictCommand
     from aetros.commands.PredictionServerCommand import PredictionServerCommand
     from aetros.commands.StartCommand import StartCommand
+    from aetros.commands.StartSimpleCommand import StartSimpleCommand
     from aetros.commands.RunCommand import RunCommand
+    from aetros.commands.AddCommand import AddCommand
+    from aetros.commands.InitCommand import InitCommand
+    from aetros.commands.IdCommand import IdCommand
+    from aetros.commands.GPUCommand import GPUCommand
+    from aetros.commands.AuthenticateCommand import AuthenticateCommand
 
     commands_dict = {
         'start': StartCommand,
+        'start-simple': StartSimpleCommand,
+        'authenticate': AuthenticateCommand,
         'predict': PredictCommand,
-        'upload-weights': UploadWeightsCommand,
         'prediction-server': PredictionServerCommand,
         'server': ServerCommand,
         'run': RunCommand,
         'api': ApiCommand,
+        'id': IdCommand,
         'push-job': PushJobCommand,
+        'pull-job': PullJobCommand,
+        'add': AddCommand,
+        'init': InitCommand,
+        'gpu': GPUCommand,
     }
 
-    log_level = 'INFO'
-    if os.getenv('DEBUG') == '1':
-        log_level = 'DEBUG'
-
-    logger = logging.getLogger('aetros-'+cmd_name)
-
-    import coloredlogs
-    coloredlogs.install(level=log_level, logger=logger)
-
     if cmd_name not in commands_dict:
         print(("Command %s not found" % (cmd_name,)))
         sys.exit(1)
 
     level = 'INFO'
-    if '-v' in args:
+    if '--debug' in args or os.getenv('DEBUG') == '1':
         level = 'DEBUG'
 
+    atty = None
+    if '1' == os.getenv('AETROS_ATTY'):
+        atty = True
+
+    import coloredlogs
     logger = logging.getLogger('aetros-'+cmd_name)
-    coloredlogs.install(level=level, logger=logger)
+    coloredlogs.install(level=level, logger=logger, isatty=atty)
     command = commands_dict[cmd_name](logger)
 
     code = command.main(cmd_args)
