@@ -239,9 +239,9 @@ def read_images_in_memory(job_model, dataset, node, trainer):
             t.start()
 
         for validation_or_training in ['validation', 'training']:
-            if os.path.isdir(path + '/' + validation_or_training):
-                for category_name in os.listdir(path + '/' + validation_or_training):
-                    if os.path.isdir(path + '/' + validation_or_training + '/' + category_name):
+            if os.path.isdir(os.path.normpath(path + '/' + validation_or_training)):
+                for category_name in os.listdir(os.path.normpath(path + '/' + validation_or_training)):
+                    if os.path.isdir(os.path.normpath(path + '/' + validation_or_training + '/' + category_name)):
 
                         if category_name not in category_map:
                             category_map[category_name] = classes_count
@@ -255,9 +255,8 @@ def read_images_in_memory(job_model, dataset, node, trainer):
 
                             classes_count += 1
 
-                        for id in os.listdir(path + '/' + validation_or_training + '/' + category_name):
-                            file_path = os.path.join(
-                                path, validation_or_training, category_name, id)
+                        for id in os.listdir(os.path.normpath(path + '/' + validation_or_training + '/' + category_name)):
+                            file_path = os.path.join(path, validation_or_training, category_name, id)
                             q.put([file_path, validation_or_training == 'validation', category_name])
                             max += 1
 
@@ -296,7 +295,7 @@ def read_images_in_memory(job_model, dataset, node, trainer):
             sys.exit(1)
 
         trainer.output_size = classes_count
-        trainer.set_info('Classes', classes)
+        trainer.set_info('classes', classes)
         trainer.classes = classes
 
         result['X_train'] = train
@@ -346,15 +345,15 @@ def read_images_keras_generator(job_model, dataset, node, trainer):
 
     if augmentation:
         train_datagen = get_image_data_augmentor_from_dataset(dataset)
-
-        if 'imageScale' not in node:
-            node['imageScale'] = 255
-
-        if float(node['imageScale']) > 0:
-            train_datagen.rescale = 1.0 / float(node['imageScale'])
     else:
         train_datagen = ImageDataGenerator()
 
+    if 'imageScale' not in node:
+        node['imageScale'] = 255
+
+    if float(node['imageScale']) > 0:
+        train_datagen.rescale = 1.0 / float(node['imageScale'])
+
     train_generator = train_datagen.flow_from_directory(
         directory=os.path.join(dataset_config['path'], 'training'),
         target_size=size,
@@ -371,12 +370,16 @@ def read_images_keras_generator(job_model, dataset, node, trainer):
         else:
             classes.append(folderName)
 
-    trainer.set_job_system_info('classes', classes)
+    trainer.set_info('classes', classes)
     trainer.classes = classes
 
     # ensure_dir(dataset_config['path'] + '/preview')
 
-    test_datagen = ImageDataGenerator(rescale=1. / 255)
+    test_datagen = ImageDataGenerator()
+
+    if float(node['imageScale']) > 0:
+        test_datagen.rescale = 1.0 / float(node['imageScale'])
+
     validation_generator = test_datagen.flow_from_directory(
         directory=os.path.join(dataset_config['path'], 'validation'),
         # save_to_dir=dataset_config['path'] + '/preview',
@@ -415,8 +418,8 @@ def read_images_keras_generator(job_model, dataset, node, trainer):
         trainer.logger.warning("Could not find any classes. Does the directory contains images?")
         sys.exit(1)
 
-    pprint(train_generator.class_indices)
-    pprint(classes)
+    trainer.logger.debug(str(train_generator.class_indices))
+    trainer.logger.debug(str(classes))
 
     return {
         'X_train': train_generator,
