@@ -7,7 +7,7 @@ import sys
 
 import os
 
-from aetros.utils import read_config
+from aetros.utils import read_home_config, setup_git_ssh
 
 
 class PushJobCommand:
@@ -29,15 +29,17 @@ class PushJobCommand:
             parser.print_help()
             sys.exit(1)
 
-        config = read_config()
+        config = read_home_config()
+
         model = parsed_args.id[0:parsed_args.id.rindex('/')]
         ref = 'refs/aetros/job/' + parsed_args.id[parsed_args.id.rindex('/')+1:]
 
-        git_dir = config['storage_dir'] + '/' + model + '.git'
+        git_dir = os.path.normpath(config['storage_dir'] + '/' + model + '.git')
 
         if not os.path.isdir(git_dir):
             self.logger.error("Git repository for model %s in %s not found." % (parsed_args.id, git_dir))
             self.logger.error("Are you in the correct directory?")
 
-        self.logger.info('Pushing ' + ref)
-        subprocess.call(['git', '--bare', '--git-dir', git_dir, 'push', 'origin', '-f', ref])
\ No newline at end of file
+        print('Pushing ' + ref + ' from ' + git_dir)
+        setup_git_ssh(config)
+        subprocess.call([config['git'], '--bare', '--git-dir', git_dir, 'push', 'origin', ref])
\ No newline at end of file
