@@ -2,16 +2,14 @@ import json
 import os
 import subprocess
 
-import signal
 import six
 from threading import Thread, Lock
-
 import time
-
 import sys
+from ruamel import yaml
 
 from aetros.api import ApiConnectionError
-from aetros.utils import invalid_json_values
+from aetros.utils import invalid_json_values, setup_git_ssh
 
 
 class GitCommandException(Exception):
@@ -34,20 +32,28 @@ class Git:
     If created in AETROS Trainer, we retrieve the initial configuration of the job using `git pull origin refs/aetros/job/<id>` and read
     its `aetros/job.json` blob of the head tree.
     """
-    def __init__(self, logger, client, git_host, storage_dir, model_name):
+    def __init__(self, logger, client, config, model_name):
         self.logger = logger
         self.client = client
-        self.git_host = git_host
-        self.storage_dir = storage_dir
+
+        self.config = config
+        self.git_host = config['host']
+        self.storage_dir = config['storage_dir']
+
         self.model_name = model_name
 
-        self.git_path = self.storage_dir + '/' + model_name + '.git'
+        self.git_path = os.path.normpath(self.storage_dir + '/' + model_name + '.git')
 
+        self.command_lock = Lock()
+        self.stream_files_lock = Lock()
         self.debug = False
         self.last_push_time = 0
         self.active_push = False
         self.index_path = None
 
+        # dirty means, the git repository has changed and need a push
+        self.dirty = False
+
         self.job_id = None
         self.online = True
         self.active_thread = False
@@ -58,29 +64,25 @@ class Git:
         self.git_batch_commit_messages = []
         self.git_last_commit = None
 
+        self.keep_stream_files = False
+
         self.streamed_files = {}
         self.store_files = {}
 
         self.prepare_index_file()
 
+        if subprocess.Popen(['git', '--version'], stdout=subprocess.PIPE).wait() > 0:
+            raise Exception("Git binary not available. Please install Git v2 first.")
+
+        self.delete_git_ssh = setup_git_ssh(config)
         self.logger.debug("GIT_SSH='" + str(os.getenv('GIT_SSH'))+"'")
         self.git_name = None
         self.git_email = None
 
-        import aetros.api
-        try:
-            user = json.loads(aetros.api.request('user-git'))
-
-            self.git_name = user['name']
-            self.git_email = user['email']
-        except ApiConnectionError as e:
-            self.online = False
-
-        if not self.git_name:
-            import getpass
-            self.git_name = getpass.getuser()
-            import socket
-            self.git_email = self.git_name + '@' + socket.gethostname()
+        import getpass
+        self.git_name = getpass.getuser()
+        import socket
+        self.git_email = self.git_name + '@' + socket.gethostname()
 
         # check if its a git repo
         if os.path.exists(self.git_path):
@@ -94,17 +96,34 @@ class Git:
 
         # check if given repo_path is current folder.
         # check its origin remote and see if model_name matches
-        origin_url = self.get_remote_url('origin')
+        self.origin_url = self.get_remote_url('origin')
 
-        if origin_url and self.git_url not in origin_url:
+        if self.origin_url and self.git_url not in self.origin_url:
             raise Exception("Given git_path (%s) points to a repository (%s) that is not the git repo of the model (%s). "
-                            "You seem to switched between aetros.com and on-premise installation. "
-                            "Please remove the Git repository." % (self.git_path, origin_url, self.git_url))
+                            "It seems you switched between aetros.com and an on-premise installation or update aetros.yml:host."
+                            "Please remove the Git repository." % (self.git_path, self.origin_url, self.git_url))
 
         if not os.path.exists(self.temp_path):
             os.makedirs(self.temp_path)
 
-        self.logger.info("Started tracking of job files in git %s for remote %s" % (self.git_path, origin_url))
+    def prepare_git_user(self):
+        """
+        Tries to read the git name and email, so all git commits have correct author.
+        Requests /api/user-git to check which user is behind the current configured ssh key.
+        """
+        import aetros.api
+        try:
+            response = aetros.api.request('user-git')
+            if response:
+                user = yaml.safe_load(response)
+
+                self.git_name = user['name']
+                self.git_email = user['email']
+            else:
+                self.go_offline()
+        except ApiConnectionError as e:
+            self.go_offline()
+
 
     def get_remote_url(self, origin_name):
         output = self.command_exec(['remote', '-v'], allowed_to_fail=True)[0].decode('utf-8').strip()
@@ -116,7 +135,7 @@ class Git:
 
     @property
     def work_tree(self):
-        return self.storage_dir + '/' + self.model_name + '/' + self.job_id
+        return os.getenv('AETROS_GIT_WORK_DIR') or os.path.normpath(self.storage_dir + '/' + self.model_name + '/' + self.job_id)
 
     @property
     def env(self):
@@ -124,17 +143,21 @@ class Git:
         if self.index_path:
             my_env['GIT_INDEX_FILE'] = self.index_path
 
+        my_env['GIT_SSH'] = os.getenv('GIT_SSH')
+
         return my_env
 
     def thread_push(self):
         while self.active_thread:
             try:
-                if self.job_id and self.online and self.active_push:
+                time.sleep(1)
+
+                if self.job_id and self.online and self.active_push and self.dirty:
+                    self.dirty = False
                     start = time.time()
                     self.command_exec(['push', '-f', 'origin', self.ref_head])
                     self.last_push_time = time.time() - start
 
-                time.sleep(1)
             except SystemExit:
                 return
             except KeyboardInterrupt:
@@ -152,6 +175,13 @@ class Git:
     def git_url(self):
         return 'git@%s:%s.git' % (self.git_host, self.model_name)
 
+    def get_base_command(self):
+        base_command = ['git', '--bare', '--git-dir', self.git_path]
+        base_command += ['-c', 'user.name=' + self.git_name]
+        base_command += ['-c', 'user.email=' + self.git_email]
+
+        return ''.join(base_command)
+
     def command_exec(self, command, inputdata=None, allowed_to_fail=False):
         interrupted = False
 
@@ -160,64 +190,77 @@ class Git:
 
         if command[0] != 'git':
             base_command = ['git', '--bare', '--git-dir', self.git_path]
-            base_command += ['-c', 'user.name=' + self.git_name]
-            base_command += ['-c', 'user.email=' + self.git_email]
+            if command[0] == 'commit-tree' or command[0] == 'commit':
+                base_command += ['-c', 'user.name=' + self.git_name]
+                base_command += ['-c', 'user.email=' + self.git_email]
             command = base_command + command
 
         p = None
         stdoutdata = ''
         stderrdata = ''
 
-        def preexec_fn():
-            # Ignore the SIGINT signal by setting the handler to the standard
-            # signal handler SIG_IGN.
-            signal.signal(signal.SIGTERM, signal.SIG_IGN)
-            signal.signal(signal.SIGINT, signal.SIG_IGN)
-
         try:
+            self.command_lock.acquire()
+
             p = subprocess.Popen(
-                command, preexec_fn=preexec_fn, bufsize=0,
+                command, bufsize=0,
                 stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=self.env
             )
 
             stdoutdata, stderrdata = p.communicate(inputdata)
         except KeyboardInterrupt:
-            try:
-                if p is not None:
-                    stdoutdata = p.stdout.read()
-                    stderrdata = p.stderr.read()
-            except:
-                pass
-            interrupted = True
-
-        input = ''
-
-        try:
-            input = inputdata.decode('utf-8')
-        except:
-            input = 'binary'
+            raise
+        finally:
+            self.command_lock.release()
 
         try:
             stderrdata = stderrdata.decode('utf-8')
-        except: pass
+        except Exception: pass
 
-        message = "Git command: " + (' '.join(command))
-        # message += "\nstdout: " + str(stdoutdata.decode('utf-8')) + ', stderr: '+ str(stderrdata)
+        self.logger.debug("Git command: " + (' '.join(command)))
 
-        self.logger.debug(message)
+        # When working on Git in several threads, sometimes it can not get the lock file, like:
+        #
+        #   fatal: Unable to create '/Users/marc/.aetros/marcj/debug:test.git/ORIG_HEAD.lock': File exists.
+        #
+        #   Another git process seems to be running in this repository, e.g.
+        #   an editor opened by 'git commit'. Please make sure all processes
+        #   are terminated then try again. If it still fails, a git process
+        #   may have crashed in this repository earlier:
+        #   remove the file manually to continue.
+        #
+        # We neeed to check for that error, and run the command again
+
+        if 'Another git process' in stderrdata:
+            time.sleep(0.3)
+            return self.command_exec(command, inputdata, allowed_to_fail)
 
         if 'Connection refused' in stderrdata or 'Permission denied' in stderrdata:
             if 'Permission denied' in stderrdata:
-                self.logger.warning("You have no permission to push to that model.")
+                self.logger.warning("You have no permission to push to that model. Make sure your SSH key is properly"
+                                    " configured.")
 
-            self.online = False
+            self.go_offline()
             self.logger.error(stderrdata)
             return '', 1, ''
 
         if not interrupted and not allowed_to_fail and p is not None and p.returncode != 0:
-            raise GitCommandException('Command failed: ' + ' '.join(command) + ', code: ' + str(p.returncode)+"\nstdout: '" + str(stdoutdata)+"',\nstderr: '" + str(stderrdata)+"', input="+str(inputdata))
+            raise GitCommandException('Command failed: ' + ' '.join(command) + ', code: ' + str(p.returncode)
+                                      +"\nstdout: '" + str(stdoutdata)
+                                      +"',\nstderr: '" + str(stderrdata)
+                                      # +"', env="+str(self.env)
+                                      +", input="+str(inputdata))
 
-        return stdoutdata, p.returncode, stderrdata
+        return stdoutdata, p.returncode if p is not None else None, stderrdata
+
+    def go_offline(self):
+        """
+        Go offline means disable all online communication and just store the data in local git.
+        """
+        if self.client:
+            self.client.go_offline()
+
+        self.online = False
 
     def prepare_index_file(self):
         """
@@ -237,15 +280,14 @@ class Git:
 
         self.logger.debug('GIT_INDEX_FILE created at ' + self.index_path)
 
-    def fetch_job(self, job_id):
+    def fetch_job(self, job_id, checkout=False):
         """
-        Fetch the current job reference (refs/aetros/job/<id>) from origin and read its tree to the current git index.
-        
-        :type job_id: str 
+        Fetch the current job reference (refs/aetros/job/<id>) from origin and (when checkout=True)read its tree to
+        the current git index and checkout into working director.
         """
         self.job_id = job_id
 
-        self.logger.info("Git fetch job reference %s" % (self.ref_head, ))
+        self.logger.debug("Git fetch job reference %s" % (self.ref_head, ))
         out, code, err = self.command_exec(['ls-remote', 'origin', self.ref_head])
 
         if code:
@@ -254,12 +296,62 @@ class Git:
 
         try:
             self.command_exec(['fetch', '-f', '-n', 'origin', self.ref_head+':'+self.ref_head])
-        except:
+        except Exception:
             self.logger.error("Could not load job information for " + job_id + '. You need to be online to start pre-configured jobs.')
             raise
 
-        self.git_last_commit = job_id
-        self.command_exec(['update-ref', self.ref_head, self.git_last_commit])
+        self.read_job(job_id, checkout)
+
+    def is_job_fetched(self, job_id):
+        try:
+            self.command_exec(['rev-parse', 'refs/aetros/job/' + job_id])[0].decode('utf-8').strip()
+            return True
+        except Exception:
+            return False
+
+    def read_job(self, job_id, checkout=False):
+        """
+        Reads head and sets self.git_last_commit, reads the tree into index,
+        and checkout the work-tree when checkout=True.
+
+        This does not fetch the job from the actual server. It needs to be in the local git already.
+        """
+        self.job_id = job_id
+
+        self.git_last_commit = self.command_exec(['rev-parse', self.ref_head])[0].decode('utf-8').strip()
+        self.logger.debug('Job ref points to ' + self.git_last_commit)
+        self.command_exec(['read-tree', self.ref_head])
+
+        if checkout:
+            self.logger.debug('Working directory in ' + self.work_tree)
+
+            # make sure we have checked out all files we have added until now. Important for simple models,
+            # so we have the actual model.py and dataset scripts.
+            if not os.path.exists(self.work_tree):
+                os.makedirs(self.work_tree)
+
+            # updates index and working tree
+            # this leaves other files in self.work_tree alone, which is necessary because this is also the working tree
+            # of files checked out by start.py (custom models)
+            self.command_exec(['--work-tree', self.work_tree, 'reset', '--hard', self.ref_head])
+
+    def read_tree(self, ref):
+        """
+        Reads the ref into the current index and points last commit_id to its head.
+
+        :param ref: the actual git reference
+        :return:
+        """
+        self.command_exec(['read-tree', ref])
+        self.git_last_commit = self.command_exec(['rev-parse', ref])[0].decode('utf-8').strip()
+
+    def restart_job(self):
+        if not self.job_id:
+            raise Exception('Could not restart unknown job. fetch_job() it first.')
+
+        self.command_exec(['update-ref', self.ref_head, self.job_id])
+        self.dirty = True
+
         self.command_exec(['read-tree', self.ref_head])
 
         # make sure we have checked out all files we have added until now. Important for simple models, so we have the
@@ -267,8 +359,10 @@ class Git:
         if not os.path.exists(self.work_tree):
             os.makedirs(self.work_tree)
 
-        self.logger.info('Working directory in ' + self.work_tree)
-        self.command_exec(['--work-tree', self.work_tree, 'checkout', self.ref_head, '.'])
+        # updates index and working tree
+        # this leaves other files in self.work_tree alone, which needs to be because this is also the working tree
+        # of files checked out by start.py (custom models)
+        self.command_exec(['--work-tree', self.work_tree, 'reset', '--hard', self.ref_head])
 
     def create_job_id(self, data):
         """
@@ -276,7 +370,7 @@ class Git:
         root commit is the actual job id. A reference is then created to the newest (head) commit of this commit history.
         The reference will always be updated once a new commit is added.
         """
-        self.add_file('aetros/job.json', json.dumps(data))
+        self.add_file('aetros/job.json', json.dumps(data, indent=4))
         tree_id = self.write_tree()
 
         self.job_id = self.command_exec(['commit-tree', '-m', "JOB_CREATED", tree_id])[0].decode('utf-8').strip()
@@ -293,7 +387,11 @@ class Git:
         if not os.path.exists(self.work_tree):
             os.makedirs(self.work_tree)
 
-        self.command_exec(['--work-tree', self.work_tree, 'checkout', self.ref_head])
+        # updates index and working tree
+        # this leaves other files in self.work_tree alone, which needs to be because this is also the working tree
+        # of files checked out by start.py (custom models)
+        self.command_exec(['--work-tree', self.work_tree, 'reset', '--hard', self.ref_head])
+        self.dirty = True
 
         return self.job_id
 
@@ -316,40 +414,75 @@ class Git:
         Stops the `git push` thread and commits all streamed files (Git.store_file and Git.stream_file), followed
         by a final git push.
         
-        This removes also the current git index file. You can not start the process again.
+        You can not start the process again.
         """
         self.active_thread = False
 
+        if self.thread_push_instance and self.thread_push_instance.isAlive():
+            self.thread_push_instance.join()
+
         with self.batch_commit('STREAM_END'):
             for path, handle in six.iteritems(self.streamed_files.copy()):
-                full_path = self.temp_path + '/stream-blob/' + self.job_id + '/' + path
-                handle.seek(0)
-                self.commit_file(path, path, handle.read())
-                handle.close()
-                os.unlink(full_path)
-            self.streamed_files = {}
+                # open again and read full content
+                full_path = os.path.normpath(self.temp_path + '/stream-blob/' + self.job_id + '/' + path)
+                self.logger.debug('Git stream end for file: ' + full_path)
+
+                del self.streamed_files[path]
+
+                # make sure its written to the disk
+                try:
+                    self.stream_files_lock.acquire()
+                    if not handle.closed:
+                        handle.flush()
+                        handle.close()
+                finally:
+                    self.stream_files_lock.release()
+
+                with open(full_path, 'r') as f:
+                    self.commit_file(path, path, f.read())
+
+                if not self.keep_stream_files:
+                    os.unlink(full_path)
 
         with self.batch_commit('STORE_END'):
             for path, bar in six.iteritems(self.store_files.copy()):
-                full_path = self.temp_path + '/store-blob/' + self.job_id + '/' + path
-                self.commit_file(path, path, open(full_path, 'r').read())
-                os.unlink(full_path)
-            self.store_files = {}
+                full_path = os.path.normpath(self.temp_path + '/store-blob/' + self.job_id + '/' + path)
+                self.logger.debug('Git store end for file: ' + full_path)
+
+                del self.store_files[path]
+
+                try:
+                    self.stream_files_lock.acquire()
+                    self.commit_file(path, path, open(full_path, 'r').read())
+                finally:
+                    self.stream_files_lock.release()
+
+                if not self.keep_stream_files:
+                    os.unlink(full_path)
 
     def clean_up(self):
+        self.logger.debug("Git: clean up")
+
         if os.path.exists(self.index_path):
             os.remove(self.index_path)
 
+        if self.delete_git_ssh:
+            if self.thread_push_instance and self.thread_push_instance.isAlive():
+                self.thread_push_instance.join()
+
+            self.delete_git_ssh()
+            self.delete_git_ssh = None
+
     def batch_commit(self, message):
         """
         Instead of committing a lot of small commits you can batch it together using this controller.
         
         Example:
-        
+
         with git.batch_commit('BATCHED'):
             git.commit_file('my commit 1', 'path/to/file', 'content from file')
             git.commit_json_file('[1, 2, 3]', 'path/to/file2', 'json array') 
-            
+
         Withing the `with` block you can use group the method calls of `commit_file` and `commit_json_file`, and every other
         method calling this two methods.
         
@@ -357,25 +490,28 @@ class Git:
         :return: with controller to be used with Python's `with git.batch_commit():`
         """
         class controlled_execution:
-            def __init__(self, job, message):
-                self.job = job
+            def __init__(self, git, message):
+                self.git = git
                 self.message = message
 
             def __enter__(self):
-                self.job.git_batch_commit = True
+                self.git.git_batch_commit = True
+                if self.git.job_id:
+                    # make sure we're always on the tip tree
+                    self.git.read_tree(self.git.ref_head)
 
             def __exit__(self, type, value, traceback):
-                self.job.git_batch_commit = False
+                self.git.git_batch_commit = False
 
                 # if nothing committed, we return early
-                if not self.job.git_batch_commit_messages: return
+                if not self.git.git_batch_commit_messages: return
 
                 commit_message = self.message
-                if self.job.git_batch_commit_messages:
-                    commit_message = commit_message + "\n\n" + "\n".join(self.job.git_batch_commit_messages)
-                self.job.git_batch_commit_messages = []
+                if self.git.git_batch_commit_messages:
+                    commit_message = commit_message + "\n\n" + "\n".join(self.git.git_batch_commit_messages)
+                self.git.git_batch_commit_messages = []
 
-                self.job.commit_index(commit_message)
+                self.git.commit_index(commit_message)
 
         return controlled_execution(self, message)
 
@@ -404,16 +540,20 @@ class Git:
         :param content: 
         :return: 
         """
+        try:
+            self.stream_files_lock.acquire()
 
-        full_path = self.temp_path + '/store-blob/' + self.job_id + '/' + path
-        if not os.path.exists(os.path.dirname(full_path)):
-            os.makedirs(os.path.dirname(full_path))
+            full_path = os.path.normpath(self.temp_path + '/store-blob/' + self.job_id + '/' + path)
+            if not os.path.exists(os.path.dirname(full_path)):
+                os.makedirs(os.path.dirname(full_path))
 
-        open(full_path, 'w+').write(data)
-        self.store_files[path] = True
+            open(full_path, 'w+').write(data)
+            self.store_files[path] = True
 
-        if self.online:
-            self.client.send({'type': 'store-blob', 'path': path, 'data': data})
+            if self.online:
+                self.client.send({'type': 'store-blob', 'path': path, 'data': data})
+        finally:
+            self.stream_files_lock.release()
 
     def stream_file(self, path):
         """
@@ -440,16 +580,19 @@ class Git:
         # open temp file
 
         # register stream file and write locally
-        # on end() git_commit that file
+        # on end() git_commit that file locally
 
         # create socket connection to server
         # stream file to server
-        # on end() send server end signal, so he can store its content in git as blob as well. A git push would detect
-        # that both sides have the same content already, except when server connection broke between start() and end().
+        # on end() send server end signal, so he can store its content in git as blob as well.
+        # A git push would detect that both sides have the same content already,
+        # except when server connection broke between start() and end().
+        # Result -> already transmitted logs/channel data (probably many MBs) won't transfered twice
+        # when doing a git-push.
 
         # return handler to write to this file
 
-        full_path = self.temp_path + '/stream-blob/' + self.job_id + '/' + path
+        full_path = os.path.normpath(self.temp_path + '/stream-blob/' + self.job_id + '/' + path)
         if not os.path.exists(os.path.dirname(full_path)):
             os.makedirs(os.path.dirname(full_path))
 
@@ -457,19 +600,27 @@ class Git:
         self.streamed_files[path] = handle
 
         class Stream():
-            def __init__(self, handle, git):
-                self.handle = handle
+            def __init__(self, git):
                 self.git = git
 
             def write(self, data):
-                try:
-                    handle.write(data)
-                    if self.git.online:
-                        self.git.client.send({'type': 'stream-blob', 'path': path, 'data': data})
-                except:
-                    pass
+                if path not in self.git.streamed_files:
+                    # already committed to server
+                    return
 
-        return Stream(handle, self)
+                try:
+                    self.git.stream_files_lock.acquire()
+                    if not handle.closed:
+                        handle.write(data)
+                        handle.flush()
+                finally:
+                    self.git.stream_files_lock.release()
+
+
+                if self.git.online:
+                    self.git.client.send({'type': 'stream-blob', 'path': path, 'data': data})
+
+        return Stream(self)
 
     def write_blob(self, content):
         return self.command_exec(['hash-object', '-w', "--stdin"], content)[0].decode('utf-8').strip()
@@ -502,20 +653,28 @@ class Git:
         blob_id = self.write_blob(content)
         self.add_index('100644', blob_id, path)
 
+    def add_local_file(self, path):
+        with open(path, 'r') as f:
+            self.add_file(path, f.read())
+
     def commit_file(self, message, path, content):
         """
         Add a new file as blob in the storage, add its tree entry into the index and commit the index.
          
-        :param message: str 
+        :param message: str
         :param path: str
         :param content: str
         :return: 
         """
-        self.add_file(path, content)
-
         if not self.git_batch_commit:
-            self.commit_index(message)
+            if self.job_id:
+                self.read_tree(self.ref_head)
+
+            self.add_file(path, content)
+
+            return self.commit_index(message)
         else:
+            self.add_file(path, content)
             self.git_batch_commit_messages.append(message)
 
     def push(self):
@@ -525,7 +684,12 @@ class Git:
         try:
             self.command_exec(['push', 'origin', '-f', self.ref_head])
             return True
-        except:
+        except Exception as e:
+            # this may fail due to:
+            #     stderr: 'remote: error: cannot lock ref 'refs/aetros/job/cc3114813659d443c8e4f9682517067a1e9ec9ff':
+            #     is at 31785d1c126b24cabd1948cba2e126912393b8e6 but expected 3f7186391884bd097c09c31567ef49718fc271a5
+            #
+            self.logger.warning(str(e))
             return False
 
     def commit_index(self, message):
@@ -536,28 +700,35 @@ class Git:
         """
         tree_id = self.write_tree()
 
-        args = ['commit-tree', tree_id]
-
-        if self.git_last_commit:
-            # if not define, it creates a new root commit
-            args += ['-p', self.git_last_commit]
+        args = ['commit-tree', tree_id, '-p', self.ref_head]
 
+        # todo, this can end in a race-condition with other processes adding commits
         self.git_last_commit = self.command_exec(args, message)[0].decode('utf-8').strip()
-
-        # update ref
         self.command_exec(['update-ref', self.ref_head, self.git_last_commit])
+        self.dirty = True
 
         return self.git_last_commit
 
-    def git_job_last_commit_sha(self):
-        if not self.job_id:
-            raise Exception('Can not fetch last commit sha, when no job_id is set.')
+    def has_file(self, path):
+        try:
+            out, code, err = self.command_exec(['cat-file', '-p', self.ref_head+':'+path])
 
-        out, code = self.command_exec(['show-ref', self.ref_head])
-        if not code:
-            return out.decode('utf-8').strip().split(' ')[0]
-        else:
-            raise Exception('Job ref not created yet. ' + self.ref_head)
+            return code == 0
+        except Exception:
+            return False
+
+    def contents(self, path):
+        """
+        Reads the given path of current ref_head and returns its content as utf-8
+        """
+        try:
+            out, code, err = self.command_exec(['cat-file', '-p', self.ref_head+':'+path])
+            if not code:
+                return out.decode('utf-8')
+        except Exception:
+            pass
+
+        return None
 
     def git_read(self, path):
         return self.command_exec(['cat-file', '-p', self.ref_head+':'+path])
