@@ -1,10 +1,14 @@
 # Copyright (c) 2014-2016, NVIDIA CORPORATION.  All rights reserved.
+# BSD 3-clause license
+
 from __future__ import absolute_import
 from __future__ import print_function
 import math
 from six.moves import range
 
 # Find the best implementation available
+from aetros.utils.pilutil import imresize
+
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -12,7 +16,6 @@ except ImportError:
 
 import numpy as np
 import PIL.Image
-import scipy.misc
 
 # Library defaults:
 #   PIL.Image:
@@ -133,7 +136,7 @@ def resize_image(image, height, width,
     width_ratio = float(image.shape[1]) / width
     height_ratio = float(image.shape[0]) / height
     if resize_mode == 'squash' or width_ratio == height_ratio:
-        return scipy.misc.imresize(image, (height, width), interp=interp)
+        return imresize(image, (height, width), interp=interp)
     elif resize_mode == 'crop':
         # resize to smallest of ratios (relatively larger image), keeping aspect ratio
         if width_ratio > height_ratio:
@@ -142,7 +145,7 @@ def resize_image(image, height, width,
         else:
             resize_width = width
             resize_height = int(round(image.shape[0] / width_ratio))
-        image = scipy.misc.imresize(image, (resize_height, resize_width), interp=interp)
+        image = imresize(image, (resize_height, resize_width), interp=interp)
 
         # chop off ends of dimension that is still too long
         if width_ratio > height_ratio:
@@ -164,7 +167,7 @@ def resize_image(image, height, width,
                 resize_width = int(round(image.shape[1] / height_ratio))
                 if (width - resize_width) % 2 == 1:
                     resize_width += 1
-            image = scipy.misc.imresize(image, (resize_height, resize_width), interp=interp)
+            image = imresize(image, (resize_height, resize_width), interp=interp)
         elif resize_mode == 'half_crop':
             # resize to average ratio keeping aspect ratio
             new_ratio = (width_ratio + height_ratio) / 2.0
@@ -174,7 +177,7 @@ def resize_image(image, height, width,
                 resize_height += 1
             elif width_ratio < height_ratio and (width - resize_width) % 2 == 1:
                 resize_width += 1
-            image = scipy.misc.imresize(image, (resize_height, resize_width), interp=interp)
+            image = imresize(image, (resize_height, resize_width), interp=interp)
             # chop off ends of dimension that is still too long
             if width_ratio > height_ratio:
                 start = int(round((resize_width - width) / 2.0))
